<?php

namespace jrosset\CliProgram\AutoDiscovery;

use Symfony\Component\Console\Command\Command;

/**
 * An application with command auto discovery
 */
trait TAutoDiscoveryApplication {
    /**
     * @var AutoDiscoverySpotsList The list of discovery spots
     */
    private AutoDiscoverySpotsList $autoDiscoverySpots;

    /**
     * The list of discovery spots
     *
     * @return AutoDiscoverySpotsList The list of discovery spots
     */
    public function getAutoDiscoverySpots (): AutoDiscoverySpotsList {
        if (!isset($this->autoDiscoverySpots)) {
            $this->autoDiscoverySpots = new AutoDiscoverySpotsList();
        }
        return $this->autoDiscoverySpots;
    }

    /**
     * Add discovered commands to application
     *
     * @return $this
     */
    public function addAutoDiscoveredCommands (): static {
        /** @var IAutoDiscoverySpot $autoDiscoverySpot */
        foreach ($this->getAutoDiscoverySpots() as $autoDiscoverySpot) {
            foreach ($autoDiscoverySpot->getCommands() as $command) {
                $this->add($command);
            }
        }
        return $this;
    }
    /**
     * Normalize command's name and aliases with auto prefixes (if any)
     *
     * @param Command $command The commande
     *
     * @return Command The command
     */
    public function normalizeCommandNameAndAliases (Command $command): Command {
        /** @var IAutoDiscoverySpot $autoDiscoverySpot */
        foreach ($this->getAutoDiscoverySpots() as $autoDiscoverySpot) {
            $command = $autoDiscoverySpot->applyAutoPrefixOnCommand($command);
        }
        return $command;
    }
}