<?php

namespace jrosset\CliProgram\Validation\Validators;

use Arrayy\Type\StringCollection;
use ReflectionEnum;
use ReflectionException;
use UnitEnum;

/**
 * An argument/option value validator based on an enumeration
 *
 * @template TEnum of UnitEnum
 * @template-implements IValidator<TEnum>
 * @template-implements TInternalValueValidator<TEnum>
 */
class EnumValidator extends BasedValidator {
    /**
     * @var ReflectionEnum The enumeration
     */
    private ReflectionEnum $enum;

    /**
     * Create a validator
     *
     * @param class-string<UnitEnum> $enumClass The enumeration class
     *
     * @throws ReflectionException If the enumeration class doesn't exist
     */
    public function __construct (string $enumClass) {
        $this->enum = new ReflectionEnum($enumClass);

        $enumCases = new StringCollection();
        foreach ($this->enum->getCases() as $enumCase) {
            $enumCases->add($enumCase->getName());
        }

        parent::__construct(new ListValidator($enumCases));
    }

    /**
     * @inheritDoc
     */
    public function getValidDefault (mixed $default): string|bool|int|float|array|null {
        if ($default instanceof UnitEnum) {
            $default = $default->name;
        }
        return $default;
    }

    /**
     * @inheritDoc
     *
     * @throws ReflectionException
     */
    public function getValue (): UnitEnum {
        $enumCase = $this->getInternalValidator()->getValue();
        return $this->enum->getCase($enumCase)->getValue();
    }
}