<?php

namespace jrosset\CliProgram\Validation\Validators;

use Arrayy\Type\StringCollection;

/**
 * An argument/option value validator based on a list of value
 *
 * @template TValue
 * @template-implements IValidator<TValue>
 * @template-implements TInternalValueValidator<TValue>
 */
class ListValidator implements IValidator {
    use TIdenticalValidDefaultValidator;
    use TInternalValueValidator;

    /**
     * @var StringCollection The list of allowed values
     */
    private StringCollection $allowedValues;

    /**
     * Create a validator
     *
     * @param StringCollection|null $allowedValues The list of allowed values
     */
    public function __construct (?StringCollection $allowedValues = null) {
        $this->setAllowedValues($allowedValues ?? new StringCollection());
    }

    /**
     * @inheritDoc
     */
    public function validate (mixed $value): bool {
        if ($this->getAllowedValues()->contains($value)) {
            $this->setValue($value);
            return true;
        }
        return false;
    }

    /**
     * The list of allowed values
     *
     * @return StringCollection The list of allowed values
     */
    public function getAllowedValues (): StringCollection {
        return $this->allowedValues;
    }
    /**
     * Set the list of allowed values
     *
     * @param StringCollection $allowedValues The list of allowed values
     *
     * @return $this
     */
    public function setAllowedValues (StringCollection $allowedValues): self {
        $this->allowedValues = $allowedValues;
        return $this;
    }
}
