<?php

namespace jrosset\CliProgram\Validation\Validators;

use DateTimeImmutable;
use DateTimeInterface;

/**
 * An argument/option value validator expecting a date (without time)
 *
 * @template-implements BasedValidator<RegexValidator>
 */
class TimeValidator extends AbstractDateTimeValidator {
    /**
     * @var DateTimeInterface The date part
     */
    private DateTimeInterface $datePart;

    /**
     * Create a validator
     *
     * @param DateTimeInterface|null $datePart The date part
     */
    public function __construct (?DateTimeInterface $datePart = null) {
        $this->setDatePart($datePart ?? (new DateTimeImmutable())->setTimestamp(0));
        parent::__construct();
    }

    /**
     * @inheritDoc
     */
    public function getValidDefault (mixed $default): string|bool|int|float|array|null {
        if ($default instanceof DateTimeInterface) {
            return $default->format('H:i:s');
        }
        return $default;
    }

    /**
     * @inheritDoc
     */
    protected function getPattern (): string {
        return /** @lang PhpRegExp */ '/^(?<hour>\d{2}):(?<minute>\d{2})(?::(?<second>\d{2}))?$/';
    }
    /**
     * @inheritDoc
     */
    public function getValue (): DateTimeInterface {
        $value = new DateTimeImmutable();
        $value = $value->setDate(
            (int)$this->getDatePart()->format('%Y'),
            (int)$this->getDatePart()->format('%m'),
            (int)$this->getDatePart()->format('%d'),
        );
        return $value->setTime(
            (int)($this->getInternalValidator()->getMatch('hour') ?? 0),
            (int)($this->getInternalValidator()->getMatch('minute') ?? 0),
            (int)($this->getInternalValidator()->getMatch('second') ?? 0),
        );
    }

    /**
     * The date part
     *
     * @return DateTimeInterface The date part
     */
    public function getDatePart (): DateTimeInterface {
        return $this->datePart;
    }
    /**
     * Set the date part
     *
     * @param DateTimeInterface $datePart The date part
     *
     * @return $this
     */
    public function setDatePart (DateTimeInterface $datePart): self {
        $this->datePart = $datePart;
        return $this;
    }
}